\input texinfo @c -*- texinfo -*-
@c %**start of header
@setfilename cxxcodingstyle.info
@settitle EPITA C++ Coding Style Guidelines

@documentlanguage en
@documentencoding ISO-8859-1
@c @smallbook
@finalout
@afourlatex
@c @setchapternewpage odd
@copying

Copyright @copyright{} 2007 @strong{EPITA} Assistants.

@end copying

@include macros.texi

@c ======================================================================

@titlepage
@title EPITA C++ Coding Style Guidelines
@subtitle Edition @today
@author EPITA Assistants
@end titlepage

@iftex
@setcontentsaftertitlepage
@end iftex

@ifnottex
@node Top
@top EPITA C++ Coding Style Guidelines
@end ifnottex

This document intends to establish a common coding style for the C++
projects of the EPITA students.

Covered topics:

@itemize @bullet
@item Naming conventions
@item Lexical layout (block level)
@item Object Oriented (OO) consideration.
@item Global layout (source file level), including header files and file
      headers
@item Project layout
@end itemize

@emph{The specifications in this document are to be known in detail by
all students.}

All submitted projects must comply @strong{exactly} with these rules.

@strong{WARNING}: Despite the fact that this document looks pretty much
like that of the C Coding Style, it is @strong{different} and specifies
@strong{different rules}. Read it carefully.

Some sections of this document are identical to the corresponding ones
of the C Coding Style. In this case, they will not be repeated and you
are asked to refer to the C Coding Style guidelines.

The rules specified in this document give you much more freedom than the
C Coding Style one, use this freedom wisely.

The
@href{http://lrde.epita.fr/~akim/compil/assignments.split/index.html,
Tiger Compiler Assignment} also has a section about Coding Style, you
@MUST{} follow it for TC. If the Tiger Assignment contradicts this
document, then it is authoritative for the Tiger Compiler.

If you are a beginner in C++, you are @strong{strongly suggested} to
entirely read @href{http://www.parashift.com/c++-faq-lite/,The C++ FAQ
light of comp.lang.c++}.

@menu
* How to read this document::
* Naming conventions::
* Preprocessor-level specifications::
* Writing style::
* Object Oriented considerations::
* Global specifications::
* Project layout::
* Differences with the C Coding Style::
* Index and Table of Contents::

@detailmenu
 --- The Detailed Node Listing ---

How to read this document

* Vocabulary::
* Rationale - intention and extension::
* Beware of the examples::

Naming conventions

* General naming conventions::
* Name capitalization::
* Typedef suffix::

Preprocessor-level specifications

* File layout::
* Preprocessor directives layout::
* Macros and code sanity::
* Comment layout::

Writing style

* Blocks::
* Declarations::
* Statements::
* Expressions::
* Control structures::
* Trailing whitespace::

Object Oriented considerations

* Lexical Rules::
* Do's and Don'ts::

Global specifications

* Casts::
* Global scope and storage::
* Code density and documentation::

Project layout

* Directory structure::

Differences with the C Coding Style

Index and Table of Contents

@end detailmenu
@end menu

@node How to read this document
@chapter How to read this document

This document adopts some conventions described in the following nodes.

@menu
* Vocabulary::
* Rationale - intention and extension::
* Beware of the examples::
@end menu

@node Vocabulary
@section Vocabulary

@cindex requirement
@cindex MUST
@cindex MUST NOT
@cindex REQUIRED
@cindex SHALL
@cindex SHOULD
@cindex RECOMMENDED
@cindex MAY
@cindex OPTIONAL
@cindex RFC 2119

These guidelines use the words @MUST{}, @MUSTNT{}, @REQUIRED{},
@SHALL{}, @SHANT{}, @SHOULD{}, @SHOULDNT{}, @RECOMMENDED{}, @MAY{} and
@OPTIONAL{} as described in RFC 2119.

Here are some reminders from RFC 2119:

@table @dfn
@item MUST
This word, or the terms @REQUIRED{} or @SHALL{}, mean that the
definition is an absolute requirement of the specification.
@item MUST NOT
This phrase, or the terms @PROHIBITED{} or @SHANT{}, mean that the
definition is an absolute prohibition of the specification.
@item SHOULD
This word, or the adjective @RECOMMENDED{}, mean that there may exist
valid reasons in particular circumstances to ignore a particular item,
but the full implications must be understood and carefully weighted
before choosing a different course.
@item SHOULD NOT
This phrase, or the phrase @NOTRECOMMENDED{}, mean that there may exist
valid reasons in particular circumstances when the particular behavior
is acceptable or even useful, but the full implications should be
understood and the case carefully weighed before implementing any
behavior described with this label.
@item MAY
This word or the adjective @OPTIONAL{}, mean that an item is truly
optional. One may choose to include the item because a particular
circumstance requires it or because it causes an interesting
enhancement.
An implementation which does not comply to an @OPTIONAL{} item @MUST{}
be prepared to be transformed to comply at any time.
@end table

@node Rationale - intention and extension
@section Rationale - intention and extension

@cindex rationale, definition

Do not mix up the intention and extension of this document.

The intention is to limit obfuscation abilities of certain students with
prior C++ experience, and make uniform the coding style of all students,
so that group work does not suffer from style incompatibilities.

The extension, that is, the precision of each ``rule'', is there to
explain how the automated coding style verification tools operate.

In brief, use your common sense and understand the intention, before
complaining about the excessive limitations of the extension.

@node Beware of the examples
@section Beware of the examples

@cindex examples, warning

Examples in these guidelines are there for illustratory purposes
@emph{only}.  When an example contradicts a specification, the
specification is authoritative.

Be warned.

As a side-note, do not be tempted to ``infer'' specifications from the
examples presented.


@node Naming conventions
@chapter Naming conventions

@cindex naming conventions
@cindex names

Names in programs must comply to several rules. They are described
in the following nodes :

@menu
* General naming conventions::
* Name capitalization::
* Typedef suffix::
@end menu

@node General naming conventions
@section General naming conventions

The recommendations provided by the C Coding Style guidelines also apply
in C++: give your @{files,variables,classes,namespaces,etc@} relevant
(English) names.

Additionally, the following rules apply:
@itemize
@item
@anchor{Stay out of reserved names}
You @MUSTNT{} define any kind of name that begins with an underscore.
They are reserved by the implementation of the compiler and standard
libraries.  This also applies for preprocessor macros (including header
guards).

@cindex class, name
@item
You @MUST{} name your classes @code{LikeThis}.

Class should be named in mixed case; for instance @code{Exp},
@code{StringExp}, @code{TempMap}, @code{InterferenceGraph} etc.  This
also applies to class templates.

@cindex public, member names
@item
You @MUST{} name @code{public} members @code{like_this}. No upper case
letters, and words are separated by an underscore.

@cindex private, member names
@cindex protected, member names
@item
You @MUST{} name @code{private} and @code{protected} members
@code{like_this_}. It is extremely convenient to have a special
convention for private and
protected members: you make it clear to the reader, you avoid gratuitous
warnings about conflicts in constructors, you leave the ``beautiful''
name available for public members etc.  We used to write
@code{_like_this}, but this goes against the standard (see @ref{Stay out
of reserved names}).
For instance, write:

@example
class IntPair
@{
public:
  IntPair (int first, int second)
    : first_ (first), second_ (second)
  @{
  @}
protected:
  int first_, second_;
@}
@end example

@item You @MUST{} declare one class @code{LikeThis} per files
@file{like-this.*}.  Each class @code{LikeThis} is implemented in a
single set of file named @file{like-this.*}.  Note that the mixed case
class names are mapped onto lower case words separated by dashes.

There can be exceptions, for instance auxiliary classes used in a single
place do not need a dedicated set of files.

@item You @MUST{} name your namespaces @code{likethis}.

@end itemize

This section is incomplete and you must follow the guidelines specified
in the section
@href{http://lrde.epita.fr/~akim/compil/assignments.split/Name-Conventions.html,
Name Conventions of Tiger Assignment}.

@node Name capitalization
@section Name capitalization

@cindex names, case of
@cindex capitalization
@cindex macro names
@cindex names, macros

Preprocessor macro names @MUST{} be entirely capitalized.

@node Typedef suffix
@section Typedef suffix

@cindex typedef, suffix

When declaring types, type names @MUST{} be suffixed with @code{_type}

@cindex @code{_type}

@example
typedef typename MyTraits<T>::res value_type;
typedef std::map<const Symbol, Entry_T> map_type;
typedef std::list<map_type> symtab_type;
@end example

@Rationale{} this is a common idiom in C++ (which happens to be used by
the @acro{stl})

@Rationale{} for not using @code{_t}: identifiers ending with
@code{_t} are reserved by @acro{posix} (beside others).

@node Preprocessor-level specifications
@chapter Preprocessor-level specifications

The global layout of files, and sections of code pertaining to the C
preprocessor (which happens to be used in C++), including file inclusion
and inclusion protection, must comply to specifications detailed in the
following sections.

@menu
* File layout::
* Preprocessor directives layout::
* Macros and code sanity::
* Comment layout::
* Header files and header inclusion::
@end menu

@node File layout
@section File layout

@cindex layout, files
@cindex lines, maximum length
@cindex CR+LF

@itemize @minus
@item
Lines @MUSTNT{} exceed 80 characters in width, including the trailing
newline character.

@item
The DOS CR+LF line terminator @MUSTNT{} be used.
Hint: do not use DOS or Windows text editors.

@cindex #if 0
@cindex disabling code blocks
@item
In order to disable large amounts of code, you @SHOULDNT{} use comments.
Use @samp{#if 0} and @samp{#endif} instead.

@Rationale{} C++ comments do not nest.

@item
Delivered project sources @MUSTNT{} contain disabled code blocks.
@end itemize

@node Preprocessor directives layout
@section Preprocessor directives layout

@cindex preprocessor, directives
@cindex directives, layout
@cindex layout, directives
@cindex #ifdef
@cindex #define

@itemize @minus
@item
The preprocessor directive mark (@samp{#}) @MUST{} appear on the first
column.

@item
Preprocessor directives following @samp{#if} and @samp{#ifdef} @MUST{}
be indented by one character:
@example
#ifndef DEV_BSIZE
# ifdef BSIZE
#  define DEV_BSIZE BSIZE
# else /* !BSIZE */
#  define DEV_BSIZE 4096
# endif /* BSIZE */
#endif /* !DEV_BSIZE */
@end example

@cindex comments, after @samp{#else} and @samp{#endif}

@item
As shown in the previous example, @samp{#else} and @samp{#endif}
@SHOULD{} be followed by a comment describing the corresponding initial
condition.

@item
The pre-processor @MUSTNT{} be used to split a token.
For instance, doing this:
@example
in\
t ma\
in ()
@{
@}
@end example
is strongly forbidden.

@item
Trigraphs and digraphs @MUSTNT{} be used.

@end itemize

@node Macros and code sanity
@section Macros and code sanity

@cindex directives, macros
@cindex preprocessor macros
@cindex macro calls
@cindex #define
@cindex capitalization

@itemize @minus
@item Preprocessor macro names @MUST{} be entirely capitalized.
@item As a general rule, preprocessor macro calls @SHOULDNT{} break code
      structure. Further specification of this point is given below.
@item Macro call @SHOULDNT{} appear where function calls wouldn't
      otherwise be appropriate. Technically speaking, macro calls
      @SHOULD{} @emph{parse} as function calls.

@Rationale{} macros should not allow for hidden syntactic
``effects''.

@item
The code inside a macro definition @MUST{} follow the specifications of
these guidelines as a whole.

@item
The code inside a macro definition @MUSTNT{} contain unbalanced
parentheses or unbalanced braces.

@item
The code inside a macro definition @MUSTNT{} break the control flow
using @code{break}, @code{continue}, @code{return} or @code{throw}.

@end itemize



@node Comment layout
@section Comment layout

@cindex comments, layout
@cindex layout, comments
@cindex comments, language
@cindex spelling mistakes

@itemize
@item
Comments @MUST{} be written in English.

@item
You @SHOULD{} prefer C-style (@samp{/** ... */}) comments for long
comments and C++/C99-style (@samp{/// ...}) comments for one-line
comments.

@item
You @MUSTNT{} document the obvious.

Don't write:
@example
/// Declaration of the Foo class.
class Foo
@{
  ...
@};
@end example
It is so obvious that you're documenting the class and the constructor
that you should not write it down.  Instead of documenting the
@emph{kind} of an entity (class, function, namespace, destructor...),
document its @emph{goal}.

@cindex imperative
@item
You @SHOULD{} use the imperative when documenting, as if you were
giving order to the function or entity you are describing.  When
describing a function, there is no need to repeat ``function'' in the
documentation; the same applies obviously to any syntactic category.
For instance, instead of:

@example
/// \brief Swap the reference with another.
/// The method swaps the two references and returns the first.
ref& swap (ref& other);
@end example

@noindent
write:

@example
/// @@brief Swap the reference with another.
/// Swap the two references and return the first.
ref& swap (ref& other);
@end example

The same rules apply to ChangeLogs.

@cindex Doxygen
@item You @SHOULD{} write your documentation in Doxygen.

@item
You @MUST{} document your extern functions, your class and their
methods, in the header that declares them, not in the file that
implement them.

@Rationale{} The clients of your interfaces will read your headers to
know how your library or module works.  They don't want to dig in the
implementation to find the documentation.

@Rationale{} The code is much more read that written and you should
favor the (many) readers instead of the (often only) implementer.

@Rationale{} Doing so leaves you some room to comment your
implementation.

@end itemize

@node Header files and header inclusion
@section Header files and header inclusion

@cindex header files
@cindex @samp{.h} files
@cindex headers, inclusion
@cindex names, header protection key
@cindex multiple inclusion protection
@cindex using namespace

@itemize @minus
@item
Header files @MUST{} be protected against multiple inclusions.

@item
Inclusion of system headers @SHOULD{} precede inclusion of local
headers.

@item
There @MUSTNT{} be any @code{using namespace} in a header.

@item
You @MUST{} document classes in their @file{*.hh} file.

@end itemize



@node Writing style
@chapter Writing style


The following sections specify various aspects of what constitutes
good programming behavior at the lexical level.

@menu
* Blocks::
* Declarations::
* Statements::
* Expressions::
* Control structures::
* Trailing whitespace::
@end menu


@node Blocks
@section Blocks


@itemize @minus

@cindex braces
@cindex blocks

@item
All braces @MUST{} be on their own line.
@multitable @columnfractions .45 .45
@item This is wrong:
@example
if (x == 3) @{
  x += 4;
@}
@end example
@tab This is correct:
@example
if (x == 3)
@{
  x += 4;
@}
@end example
@end multitable

@item
As an exception, you are allowed to add a comment after a closing brace
in order to specify what is getting closed.
@example
namespace foo
@{
  // FIXME: Add 500 lines here.
@} // namespace foo
@end example
But @SHOULDNT{} state the obvious.

@item
As another exception, you are allowed to put the opening brace after a
@code{try} or a @code{do} on the same line. This code is correct:
@example
int foo ()
@{
  int c = 0;
  int r = 0;

  do @{
    try @{
      r = bar (++c);
    @}
    catch (...)
    @{
      r = 0;
    @}
  @} while (c < 42 && !r);
  return r;
@}
@end example

@item
Closing braces @MUST{} appear on the same column as the corresponding
opening brace.

@item
The text between two braces @MUST{} be indented by a fixed, homogeneous
amount of whitespace. This amount @SHOULD{} be 2 or 4 spaces.

@item
Opening braces @SHOULD{} appear on the same column as the text before.
However, they @MAY{} be shifted with an indentation level after control
structures, in which case the closing brace @MUST{} be shifted with the
same offset.

@end itemize


@node Declarations
@section Declarations

@subsection Alignment

@cindex pointerness, alignment of declarations
@cindex alignment, pointer declarations

Pointers and references are part of the type, and @MUST{} be put near the
type, not near the variable.
@example
const char* p;  // not `const char *p;'
std::string& s; // not `std::string &s;'
void* magic (); // not `void *magic();'
@end example

@subsection Declarations

@itemize @minus

@cindex declarations, one per line

@item
There @MUST{} be only one declaration per line.

@cindex multiple declarations
@cindex variables, multiple per line

@cindex inner declarations

@item
Inner declarations (i.e. at the start of inner blocks) are
@RECOMMENDED{}. Variables @SHOULD{} be declared close to their first use
and their scope must be kept as small as possible.

@cindex initialization
@cindex declarations with initialization

@item
Variables @SHOULD{} be initialized at the point of declarations.

Hint: Your compiler can help you to detect uninitialized local
variables.

@end itemize


@node Statements
@section Statements

@cindex statement, multiple per line

@itemize @minus
@item
A single line @MUSTNT{} contain more than one statement.
@multitable @columnfractions .45 .45
@item This is wrong:
@example
x = 3; y = 4;
@end example
@tab This is correct:
@example
x = 3;
y = 4;
@end example
@end multitable

@cindex commas

@item
Commas @MUSTNT{} be used on a line to separate statements.

@item
The commas @MUST{} be followed by a space character.

@cindex semicolons

@item
The semicolons @MUST{} be followed by a newline, and @MUSTNT{} be
preceded by a whitespace, except if alone on the line.

@cindex keywords, followed by whitespace
@cindex @code{return}
@cindex @code{continue}
@cindex @code{break}

@item
Keywords @MUST{} be followed by a single whitespace, @emph{except}
those without arguments.

@cindex @code{goto}

@item
The @code{goto} statement @MUSTNT{} be used.

Of course, this rule is not applicable on source files generated by external
tools (like @command{bison} or @command{flex}).

@cindex @code{asm}

@item
The @code{asm} declaration @MUSTNT{} be used.

@end itemize


@node Expressions
@section Expressions

@cindex expressions, padding with spaces
@cindex spaces, within expressions
@cindex operators, padding

@itemize @minus
@item
All binary and ternary operators @MUST{} be padded on the left and right by one
space, @strong{including} assignment operators.

@item
Prefix and suffix operators @MUSTNT{} be padded, neither on the left nor
on the right.


@item
When necessary, padding is done with a single whitespace.

@item
The @code{.}, @code{->}, @code{::}, @code{operator[]} and
@code{operator()} operators @MUSTNT{} be padded.
@multitable @columnfractions .45 .45
@item This is wrong:
@example
  x+=10*++x;
  y=a?b:c;
@end example
@tab This is correct:
@example
  x += 10 * ++x;
  y = a ? b : c;
@end example
@end multitable

@item
In C++ you @SHOULD{} write @code{0} instead of @code{NULL} for NULL pointers.

@item
There @MUST{} be a space between the function name arguments.
@example
int
foo (int n)
@{
  return bar () + n;
@}
@end example

@item
Expressions @MAY{} span over multiple lines. When a line break occurs
within an expression, it @MUST{} appear just @strong{before} a binary
operator, in which case the binary operator @MUST{} be indented with
@emph{at least} an extra indentation level.
@example
std::cout << "Hello!" << std::endl
  << "World!\n" << std::endl;
@end example


@end itemize


@node Control structures
@section Control structures

@subsection General rules

@cindex keywords, followed by whitespace
@cindex control structure keywords

@itemize @minus

@cindex do
@cindex if
@cindex typeid
@cindex sizeof
@cindex case
@cindex catch
@cindex switch
@cindex template
@cindex for
@cindex throw
@cindex while
@cindex try

@item
The following keywords @MUST{} be followed by a whitespace:
@example
do if typeid sizeof case catch switch template for throw while try
@end example

@multitable @columnfractions .45 .45
@item This is wrong:
@example
if(x == 3)
  foo3();
@end example
@tab
This is correct:
@example
if (x == 3)
  foo3 ();
@end example
@end multitable

@cindex template, specialization
@cindex template<>

@item
As an exception of the previous rule, you are allowed (even
@RECOMMENDED{}) to write @code{template<>} for explicit
specializations.
@example
template<>
class Foo<int>
@{
  ...
@};
@end example

@item
Each of the three parts of the @code{for} construct @MAY{} be empty.
Note that more often than not, the @code{while} construct better
represents the loop resulting from a @code{for} with an empty initial
part.

@multitable @columnfractions .45 .45
@item These are wrong:
@example
for (;;) ;

for ( ; ; ) ;
@end example
@tab This is correct:
@example
for (;;)
  ;
@end example
@end multitable

@end itemize

@subsection Loops, general rules

@itemize @minus

@cindex @samp{for}, empty body
@cindex @samp{while}, empty body
@cindex empty loop body

@item
To emphasize the previous rules, single-line loops
(@samp{for} and @samp{while}) @MUST{} have their terminating
semicolon on the following line.
@multitable @columnfractions .6
@item This is wrong:
@example
for (len = 0; *str; ++len, ++str);
@end example
@item These are correct:
@example
for (len = 0; *str; ++len, ++str)
  ;
@end example
@end multitable


@end itemize

@node Trailing whitespace
@section Trailing whitespace

@cindex trailing whitespace
@cindex whitespace, at the end of a line
@cindex CR+LF
@cindex tabulations

@itemize @minus
@item
There @MUSTNT{} be any whitespace at the end of a line.

@Rationale{} although this whitespace is usually not visible, it
clobbers source code with useless bytes.

@item
While it is not a requirement, contiguous whitespace @MAY{} be merged
with tabulation marks, assuming 8-space wide tabulations.

@item
(Reminder, @pxref{File layout}) The DOS CR+LF line terminator @MUSTNT{}
be used. Hint: do not use DOS or Windows text editors.

@end itemize

@node Object Oriented considerations
@chapter Object Oriented considerations

@menu
* Lexical Rules::
* Do's and Don'ts::
@end menu


@node Lexical Rules
@section Lexical Rules

@itemize

@cindex visibility
@cindex public
@cindex protected
@cindex private
@item
You @MUST{} order class members by visibility first.

When declaring a class, start with @code{public} members, then
@code{protected}, and last @code{private} members.  Inside these groups,
you are invited to group by category, i.e., methods, types, and members
that are related should be grouped together.

@Rationale{} People reading your class are interested in its interface
(that is, its @code{public} part).  @code{private} members should not
even be visible in the class declaration (but of course, it is mandatory
that they be there for the compiler), and therefore they should be
``hidden'' from the reader.
This is an example of what should @strong{not} be done:

@example
class Foo
@{
public:
  Foo (std::string, int);
  virtual ~Foo ();

private:
  typedef std::string string_type;
public:
  std::string bar_get () const;
  void bar_set (std::string);
private:
  string_type bar_;

public:
  int baz_get () const;
  void baz_set (int);
private:
  int baz_;
@}
@end example
@noindent
instead, write:
@example
class Foo
@{
public:
  Foo (std::string, int);
  virtual ~Foo ();

  std::string bar_get () const;
  void bar_set (std::string);

  int baz_get () const;
  void baz_set (int);

private:
  typedef std::string string_type;
  string_type bar_;
  int baz_;
@}
@end example
@noindent
and add useful Doxygen comments.

@cindex inheritance
@item You @SHOULD{} keep superclasses on the class declaration line.
Leave a space at least on the right hand side of the colon.  If there is
not enough room to do so, leave the colon on the class declaration line.
@example
class Derived: public Base
@{
  // ...
@};

/// Object function to compare two Temp*.
struct temp_ptr_less:
  public std::binary_function<const Temp*, const Temp*, bool>
@{
  bool operator() (const Temp* s1, const Temp* s2) const;
@};
@end example

@cindex virtual
@item
You @MUST{} repeat @code{virtual} in subclass declarations.
If a method was once declared @code{virtual}, it remains virtual.
Nevertheless, as an extra bit of documentation to your fellow
developers, repeat this @code{virtual}:
@example
class Base
@{
  public:
    // ...
    virtual foo ();
@};

class Derived: public Base
@{
  public:
    // ...
    virtual foo ();
@};
@end example

@item You @MUSTNT{} leave spaces between template name and effective
parameters:
@example
std::list<int> l;
std::pair<std::list<int>, int> p;
@end example

@noindent
with a space after the comma, and of course between two closing
@samp{>}:

@example
std::list<std::list<int> > ls;
@end example

These rules apply for casts:

@example
// Come on baby, light my fire.
int* p = static_cast<int*> (42);
@end example

@item
You @MUST{} leave one space between @sc{template} and formal parameters.
Write

@example
template <class T1, class T2>
struct pair;
@end example

@noindent
with one space separating the keyword @code{template} from the list of
formal parameters.  For explicit specializations you @MAY{} (and are
@RECOMMENDED{} to) write @code{template<>}:

@example
template<>
struct pair<int, int>
@{ ... @};
@end example

@item
You @MUST{} leave a space between function name and arguments.
The @code{()} operator is not a list of arguments.
@example
class Foo
@{
  public:
    Foo ();
    virtual ~Foo ();
    bool operator() (int n);
@};
@end example

@item
You @MUSTNT{} specify @code{void} if your function or method does not take any
argument. The C++ way of doing is to simply write that your function does not
take any argument.

@strong{Don't} do this:
@example
int main(void)
@{
  return 0;
@}
@end example
Write this instead (without the comments, obviously):
@example
int
main () // Not like C: main can't be passed any argument
@{
@} // In C++, main implicitly returns 0
@end example

@item
You @MAY{} specify the return type of a function or method on its own
line. This is the @RECOMMENDED{} way described in the @acro{gnu} Coding
Standards.
@example
int
foo (int n)
@{
  return bar (n);
@}
@end example

@item
You @MUST{} put initializations below the constructor declaration.

Don't put initializations or constructor invocations on the same line
as the constructor.  As a matter of fact, you @MUSTNT{} even leave
the colon on that line.  Instead of @code{A::A (): B (), C()}, write
either:

@example
A::A ()
  : B (),
    C ()
@{
@}
@end example

@noindent
or

@example
A::A ()
  : B (), C ()
@{
@}
@end example

@Rationale{} the initialization belongs more to the body of the
constructor than its signature.  And when dealing with exceptions
leaving the colon above would yield a result even worse than the
following.
@example
A::A ()
try
  : B (),
    C ()
@{
@}
catch (...)
@{
@}
@end example


@end itemize

@node Do's and Don'ts
@section Do's and Don'ts

@itemize

@item
Read @href{http://www.parashift.com/c++-faq-lite/,The C++ FAQ light of
comp.lang.c++}.  This is by far the most useful document you can read while
learning C++.

@item
Trust your compiler. Let it be your friend. Use the warnings:
@command{-Wall -W}.

@item
Read
@href{http://lrde.epita.fr/~akim/compil/assignments.split/Use-of-STL.html,
what Tiger Assignment says about the use of @acro{stl}}.

@item
You @SHOULD{} declare your constructors with one argument @code{explicit}
unless you know you want them to trigger implicit conversions.

@item
You @MUST{} initialize the attributes of an object with the
constructor's member initialization line whenever it's possible.

Don't write:
@example
class Foo
@{
public:
  Foo (int i);
private:
  int i_;
@};

Foo::Foo (int i)
@{
  i_ = i;
@}
@end example

But instead write:

@example
class Foo
@{
public:
  Foo (int i);
private:
  int i_;
@};

Foo::Foo (int i)
  : i_ (i)
@{
@}
@end example

@item
When a constructor fails, it @MUST{} throw an exception instead of
leaving the newly created object in a zombie state (see
@href{http://www.parashift.com/c++-faq-lite/exceptions.html#faq-17.2,
this link}).

@item
A destructor @MUSTNT{} fail, that is, it @MUSTNT{} throw an exception
(see
@href{http://www.parashift.com/c++-faq-lite/exceptions.html#faq-17.3,
this link}).

@item
Your destructor @MUST{} be @code{virtual} if there is at least one
virtual method in your class
(see
@href{http://www.parashift.com/c++-faq-lite/virtual-functions.html#faq-20.7,
When should my destructor be virtual?}).

@item
You @MUSTNT{} overload @code{operator,}, @code{operator||} and
@code{operator&&}.

@item
You @SHOULDNT{} overload @code{operator&}.

@item
You @MUSTNT{} use @code{friend}s to circumvent bad design.

@item
You @MUSTNT{} throw literal constants.
You @SHOULDNT{} throw anything else than a temporary object.
In particular, you @SHOULDNT{} throw objects created with @code{new}.
See
@href{http://www.parashift.com/c++-faq-lite/exceptions.html#faq-17.6,
this FAQ}.

@item
You @MUSTNT{} put parenthesis after a @code{throw} because some compilers
(including some versions of @acro{gcc}) will reject your code. Here is an
example of what should not be done:
@example
int main ()
@{
  throw (Foo ());
@}
@end example

@item
You @SHOULD{} catch by reference.

@item
Once again, you @SHOULD{} @strong{really} read
@href{http://www.parashift.com/c++-faq-lite/,The C++ FAQ light of
comp.lang.c++}.  This is by far the most useful document you can read while
learning C++.

@end itemize

@node Global specifications
@chapter Global specifications

Some general considerations about the C++ sources of a project
are specified in the following sections.


@menu
* Casts::
* Global scope and storage::
* Code density and documentation::
@end menu




@node Casts
@section Casts

@cindex casts, do not use
@cindex transtyping, do not use

@itemize
@item
As a general rule, C casts @MUSTNT{} be used.

@cindex dynamic_cast
@cindex static_cast
@cindex const_cast
@cindex reinterpret_cast
Use the C++ casts instead: @code{dynamic_cast}, @code{static_cast},
@code{const_cast}, @code{reinterpret_cast}.

@Rationale{} good programming behavior includes proper type handling.

@item
Use of @code{reinterpret_cast} is @NOTRECOMMENDED{}.

@end itemize

@node Global scope and storage
@section Global scope and storage

@cindex object, with static storage
@cindex static storage, objects
@cindex global, object
You @SHOULDNT be using global objects or objects with static storage.
Whenever you do, you must be aware that their construction order is
subject to the so-called @emph{static initialization order fiasco}
(@href{http://www.parashift.com/c++-faq-lite/ctors.html#faq-10.12,see
this link}).

@node Code density and documentation
@section Code density and documentation

@itemize @minus
@item
(Reminder, @pxref{File layout})
Lines @MUSTNT{} exceed 80 characters in width, including the trailing
newline character.

@cindex comments, before functions only
@cindex Doxygen

@item
Function declarations @SHOULD{} be preceded by a Doxygen comment
explaining the purpose of the function. This explanatory comment
@SHOULD{} contain a description of the arguments, the error cases, the
return value (if any) and the algorithm implemented by the function.

@item
Function bodies @MAY{} contain comments and blank lines.

@cindex line count within function bodies

@item
Functions' body @MUSTNT{} contain more than @strong{50} lines. The
enclosing braces are excluded from this count as well as the blank lines
and comments.

@end itemize



@node Project layout
@chapter Project layout

Specifications in this chapter are to be altered (most often
relaxed) by the assistants on a per-project basis. When in
doubt, follow these guidelines.

@menu
* Directory structure::
@end menu

@node Directory structure
@section Directory structure

This section is identical to the corresponding one described in the C
Coding Style.

Note, however, that many people tend to use @command{make tests} instead
of @command{make check} or to put their tests in a directory named
@code{test} or @code{check} whereas it @SHOULD{} be named @code{tests}
in order to comply with the @acro{gnu} Coding Standards.

@node Differences with the C Coding Style
@chapter Differences with the C Coding Style

This is a non exhaustive list of the differences between the guidelines
provided by this document and that provided by the C Coding Style.

@itemize @bullet

@item
You don't have to and even @MUSTNT{} add @code{s_}, @code{u_},
@code{e_}, @code{t_} before your type names.

@item
Global variables don't have to be prefixed by @code{g_}.

@item
You don't have to and even @SHOULDNT{} add the EPITA header to all your
files. It is deprecated and useless. SVN does a better and more useful
job if you want to track modifications.

@item
You can write your comments the way you want instead of having to stick
to
@example
/*
** Comment.
*/
@end example

@item
You don't have to and even @SHOULDNT{} align everything (declarations,
arguments, local variable names, member fields in structures, enums or
unions, etc).

@item 
Pointerness is to be expressed as part of the type (@code{int* p})
despite the (sad) fact that @code{int* p, q} declares a pointer and an
integer. You @MUSTNT{} declare two things on the same line anyway.

@item
When declaring @strong{and} initializing your variable you can
initialize them the way you want, not only with simple expressions as
required by the C Coding Style.

@item
The @code{return} value doesn't have to be in parentheses. Actually, it
@SHOULDNT unless adding parentheses makes it easier to read (e.g, because
it spans over more than one line).

@item
Almost all parentheses must be preceded by a whitespace.

@item
When breaking a long expression so that it spans over more than one
line, the break must occur @strong{before} a binary operator, not after.
This is what the @acro{gnu} Coding Standards suggest (and justifies).

@item
You are allowed and advised to write @code{else if} on the same line.

@item
Functions can take more than 4 arguments although it is usually not
necessary.

@item
You are not limited to any number of functions per file. A common
practice in C++ is to define all the methods of a class in the same
file.

@item
Function bodies @MAY{} contain up to 50 (useful) lines maximum instead
of 25.

@end itemize

@node Index and Table of Contents
@unnumbered Index and Table of Contents

@printindex cp

@contents


@bye
